// -----------------------------------------------------------------------
// Litmus - DKFZ SIDT unit testing library
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostic and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See litmusCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/litmus/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 4433 $ (last changed revision)
// @date    $Date: 2013-01-22 14:38:20 +0100 (Di, 22 Jan 2013) $ (last change date)
// @author  $Author: floca $ (last changed by)
*/


#ifndef __LIT_TEST_REPORTER_H
#define __LIT_TEST_REPORTER_H

#include "litTestLabel.h"

namespace lit {

    /*! @brief Base class for test reporters.
     * Test reporter are used (e.g. by TestResults) to generate outputs of the testing.
     * @ingroup Testing*/
    class LitmusCommon_EXPORT TestReporter
    {
    public:
      typedef TestLabel LabelType;

      virtual ~TestReporter();
      TestReporter();

      virtual void reportComment(const StringType& comment) = 0;
      virtual void reportTestStart(const LabelType& label) = 0;
      virtual void reportFailure(const LabelType& label, const StringType& failure) = 0;
      virtual void reportSuccess(const LabelType& label, const StringType& success) = 0;
      virtual void reportSummary(unsigned long totalTestCount, unsigned long failureCount, double secondsElapsed) = 0;
    private:
      TestReporter(TestReporter const&); //purposely not implemented
      TestReporter& operator =(TestReporter const&); //purposely not implemented

    };

}
#endif
